#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_GET_TEMP 1

uint8_t in_c(uint8_t pin)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นอินพุท
    DDRC &= ~(1<<pin);
    // ดึงให้ขามีลอจิก 1 หากขาถูกปล่อยลอยไว้ 
    PORTC |= (1<<pin);
    // อ่านสถานะลอจิกของขา
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(uint8_t pin, uint8_t val)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นเอาท์พุท
    DDRC |= 1<<pin;
    // เซ็ตลอจิกของขาเป็น 1 ถ้า val ไม่ใช่ศูนย์ ไม่เช่นนั้นเซ็ตให้เป็น 0
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[2];  /* ข้อมูลนี้ต้องไม่ถูกเขียนทับหลังจาก usbFunctionSetup รีเทิร์น */


    if (rq->bRequest == VENDOR_RQ_GET_TEMP)
    {
	uint16_t temp;
	temp = read_adc(4);
	dataBuffer[0] = temp/256;
	dataBuffer[1] = temp%256;
        usbMsgPtr = dataBuffer;   /* ระบุว่าข้อมูลส่งกลับอยู่ใน dataBuffer */
        return 2;                 /* ความยาวข้อมูลส่งกลับเท่ากับ 1 */
    }
    return 0;   /* ไม่รู้จักคำร้องขอ ไม่ส่งข้อมูลกลับ */
}



int main()
{
	DDRC  = 0b00000111;  // PC0..PC2 = Output, PC3..PC5 = Input
	PORTC = 0b00000001;  // PC7..PC0 = 0
	DDRD  = 0b00001000;
	PORTD = 0b00000000;
    	usbInit();
    	usbDeviceDisconnect();
    	_delay_ms(300);     /* fake USB disconnect for > 250 ms */
    	usbDeviceConnect();
    	sei();              /* enable interrupts */
    	while (1)           /* main event loop */
    	{                
        	usbPoll();      /* คำสั่งนี้ต้องถูกเรียกอย่างน้อยที่สุดทุก ๆ 50ms */
    	}
    	return 0;
}

